//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import Flutter
import NevisMobileAuthentication

struct AuthCloudApiRegistrationMethodHandler: MethodHandler {
	func execute(using client: MobileAuthenticationClient, with message: ChannelInMessage, result: @escaping FlutterResult) throws {
		let message: AuthCloudApiRegistrationMessage = try validate(message: message)
		let operation = AuthCloudApiRegistrationOperation(operationId: message.operationId,
		                                                  pinPolicy: message.pinPolicy)
		OperationCache.shared.put(operation, using: message.operationId)

		let authCloudApiRegistration = client.operations.authCloudApiRegistration
			.authenticatorSelector(operation.authenticatorSelector)
			.pinEnroller(operation.pinEnroller)
			.biometricUserVerifier(operation.biometricUserVerifier)
			.onSuccess { operation.onSuccess(nil) }
			.onError(operation.onError)

		if let enrollResponse = message.enrollResponse {
			authCloudApiRegistration.enrollResponse(enrollResponse)
		}
		else if let appLinkUri = message.appLinkUri {
			authCloudApiRegistration.appLinkUri(appLinkUri)
		}

		if let deviceInformation = message.deviceInformation {
			authCloudApiRegistration.deviceInformation(deviceInformation)
		}

		if let allowDevicePasscodeAsFallback = message.allowDevicePasscodeAsFallback {
			authCloudApiRegistration.allowDevicePasscodeAsFallback(allowDevicePasscodeAsFallback)
		}

		authCloudApiRegistration.execute()
		result(EmptyResultMessage())
	}
}
