//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import Flutter
import NevisMobileAuthentication

struct AuthenticationMethodHandler: MethodHandler {
	func execute(using client: MobileAuthenticationClient, with message: ChannelInMessage, result: @escaping FlutterResult) throws {
		let message: AuthenticationMessage = try validate(message: message)
		let operation = AuthenticationOperation(operationId: message.operationId)
		OperationCache.shared.put(operation, using: message.operationId)
		let authentication = client.operations.authentication
			.username(message.username)
			.authenticatorSelector(operation.authenticatorSelector)
			.pinUserVerifier(operation.pinUserVerifier)
			.biometricUserVerifier(operation.biometricUserVerifier)
			.onSuccess(operation.onSuccess)
			.onError(operation.onError)

		if let typedSessionProvider = message.sessionProvider {
			authentication.sessionProvider(typedSessionProvider.wrapped)
		}
		
		if let retryPolicy = message.retryPolicy {
			authentication.retryPolicyObtainingAuthorizationProvider(retryPolicy.wrapped)
		}

		authentication.execute()
		result(EmptyResultMessage())
	}
}
