//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import Flutter
import NevisMobileAuthentication

struct OutOfBandRegistrationMethodHandler: MethodHandler {
	func execute(using _: MobileAuthenticationClient, with message: ChannelInMessage, result: @escaping FlutterResult) throws {
		let message: OutOfBandRegistrationMessage = try validate(message: message)
		let operation: OutOfBandRegistrationOperation = try OperationCache.shared.read(by: message.operationId)

		operation.update(operationId: message.operationId,
		                 pinPolicy: message.pinPolicy)
		operation.registration
			.authenticatorSelector(operation.authenticatorSelector)
			.pinEnroller(operation.pinEnroller)
			.biometricUserVerifier(operation.biometricUserVerifier)
			.onSuccess { operation.onSuccess(nil) }
			.onError(operation.onError)

		if let deviceInformation = message.deviceInformation {
			operation.registration.deviceInformation(deviceInformation)
		}

		if let allowDevicePasscodeAsFallback = message.allowDevicePasscodeAsFallback {
			operation.registration.allowDevicePasscodeAsFallback(allowDevicePasscodeAsFallback)
		}

		operation.registration.execute()
		OperationCache.shared.put(operation, using: message.operationId)
		result(EmptyResultMessage())
	}
}
