//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import Flutter
import NevisMobileAuthentication

struct RegistrationMethodHandler: MethodHandler {
	func execute(using client: MobileAuthenticationClient, with message: ChannelInMessage, result: @escaping FlutterResult) throws {
		let message: RegistrationMessage = try validate(message: message)
		let operation = RegistrationOperation(operationId: message.operationId,
		                                      pinPolicy: message.pinPolicy)
		OperationCache.shared.put(operation, using: message.operationId)
		let registration = client.operations.registration
			.username(message.username)
			.authenticatorSelector(operation.authenticatorSelector)
			.pinEnroller(operation.pinEnroller)
			.biometricUserVerifier(operation.biometricUserVerifier)
			.onSuccess { operation.onSuccess(nil) }
			.onError(operation.onError)

		if let deviceInformation = message.deviceInformation {
			registration.deviceInformation(deviceInformation)
		}

		if let typedAuthorizationProvider = message.authorizationProvider {
			registration.authorizationProvider(typedAuthorizationProvider.wrapped)
		}
		
		if let allowDevicePasscodeAsFallback = message.allowDevicePasscodeAsFallback {
			registration.allowDevicePasscodeAsFallback(allowDevicePasscodeAsFallback)
		}

		registration.execute()
		result(EmptyResultMessage())
	}
}
