// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/on_valid_credentials_provided_test_data.dart';
import '../util/ios_test_init.dart';
import '../util/method_channel_native_call_mock.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    const MethodChannel channel = MethodChannel(
      CommonTestData.methodChannelName,
    );

    group("Given onValidCredentialsProvided is called from the native side",
        () {
      const methodName = 'onValidCredentialsProvided';
      late MockMethodChannelMethodCallCache mockCache;
      group("When the incoming parameters are correct", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockOperation.updateHandler(any)).thenReturn(mockOperation);
          when(mockOperation.onValidCredentialsProvided(any)).thenReturn(null);

          await channel.invokeMockMethod(
            methodName,
            OnValidCredentialsProvidedTestData.correctArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockOperation.onValidCredentialsProvided(any),
          ]);
        });
      });

      group("When the operationId is missing", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.wrongId;
          when(mockCache.getOperation(operationId)).thenThrow(
              PlatformChannelException(
                  error: PlatformChannelError.noOperationFoundError(
                      operationId: operationId)));

          try {
            await channel.invokeMockMethod(
              methodName,
              OnValidCredentialsProvidedTestData.wrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(
              e.message,
              CommonTestData.platformChannelExceptionMatcher,
            );
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verify(mockCache.getOperation(operationId));
        });
      });
    });
  });
}
