// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/authentication_test_data.dart';
import '../test_data/common_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache mockCache;
    late MethodChannelMock channel;

    group("Given authentication is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        mockCache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = mockCache;
        channel = MethodChannelMock(
          method: 'authenticate',
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        test("Then result is success", () async {
          final sessionProvider = MockSessionProvider();
          final authenticatorSelector = MockAuthenticatorSelector();
          final pinUserVerifier = MockPinUserVerifier();
          final biometricUserVerifier = MockBiometricUserVerifier();
          final fingerprintUserVerifier = MockFingerprintUserVerifier();
          final operation = MockAuthenticationPlatformOperation();
          when(mockCache.createOperation(any)).thenReturn(operation);
          when(sessionProvider.toJson()).thenReturn({});
          await sdk.operations.authenticate(
              operationId: CommonTestData.operationId,
              username: CommonTestData.username1,
              sessionProvider: sessionProvider,
              authenticatorSelector: authenticatorSelector,
              pinUserVerifier: pinUserVerifier,
              biometricUserVerifier: biometricUserVerifier,
              fingerprintUserVerifier: fingerprintUserVerifier,
              onSuccess: (provider) {},
              onError: (error) {});
          expect(channel.mappedArgs, AuthenticationTestData.correctArguments);
          verify(
            mockCache.createOperation(
                argThat(isA<AuthenticationPlatformOperation>())),
          );
        });
      });
    });
  });
}
