// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/change_pin_test_data.dart';
import '../test_data/common_test_data.dart';
import '../test_data/pin_policy_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late MockMethodChannelMethodCallCache mockCache;
    late MethodChannelMock channel;

    group("Given changePin is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        mockCache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = mockCache;
        channel = MethodChannelMock(
          method: 'changePin',
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        test("Then result is success", () async {
          final pinChanger = MockPinChanger();
          final operation = MockChangePinPlatformOperation();
          when(mockCache.createOperation(any)).thenReturn(operation);
          when(pinChanger.pinPolicy).thenReturn(PinPolicyTestData.pinPolicy);

          await sdk.operations.changePin(
            operationId: CommonTestData.operationId,
            username: CommonTestData.username1,
            pinChanger: pinChanger,
            onSuccess: () {},
            onError: (error) {},
          );

          expect(channel.mappedArgs, ChangePinOutTestData.correctArguments);
          verify(
            mockCache
                .createOperation(argThat(isA<ChangePinPlatformOperation>())),
          );
        });
      });
    });
  });
}
