// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/device_information_test_data.dart';
import '../test_data/oob_registration_test_data.dart';
import '../test_data/pin_policy_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache mockCache;
    late MethodChannelMock channel;

    group("Given outOfBandRegister is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        mockCache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = mockCache;
        channel = MethodChannelMock(
          method: 'oobRegister',
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        test("Then result is success", () async {
          final deviceInformation = MockDeviceInformation();
          final authenticatorSelector = MockAuthenticatorSelector();
          final pinEnroller = MockPinEnroller();
          final biometricUserVerifier = MockBiometricUserVerifier();
          final fingerprintUserVerifier = MockFingerprintUserVerifier();
          final operation = MockOutOfBandRegistrationPlatformOperation();
          when(mockCache.createOperation(any)).thenReturn(operation);
          when(deviceInformation.toJson())
              .thenReturn(DeviceInformationTestData.deviceInformationMap);
          when(pinEnroller.pinPolicy).thenReturn(PinPolicyTestData.pinPolicy);

          await sdk.operations.outOfBandRegister(
            operationId: CommonTestData.operationId,
            deviceInformation: deviceInformation,
            authenticatorSelector: authenticatorSelector,
            pinEnroller: pinEnroller,
            biometricUserVerifier: biometricUserVerifier,
            fingerprintUserVerifier: fingerprintUserVerifier,
            onSuccess: () {},
            onError: (error) {},
          );

          expect(channel.mappedArgs, OobRegistrationTestData.correctArguments);
          verify(
            mockCache.createOperation(
                argThat(isA<OutOfBandRegistrationPlatformOperation>())),
          );
        });
      });
    });
  });
}
